'use strict';

(function () {
    angular
        .module('cybersponse')
        .factory('aiAssistantService', aiAssistantService);

    aiAssistantService.$inject = ['$q', 'API', '$resource', '$interval', '$rootScope', 'playbookService', 'websocketService', 'connectorService'];

    function aiAssistantService($q, API, $resource, $interval, $rootScope, playbookService, websocketService, connectorService) {

        var service = {
            getAllPlaybooks: getAllPlaybooks,
            checkPlaybookExecutionCompletionByPolling: checkPlaybookExecutionCompletionByPolling,
            checkPlaybookExecutionCompletion: checkPlaybookExecutionCompletion,
            copyWithoutBotGeneral: copyWithoutBotGeneral,
            connectorHealthCheck: connectorHealthCheck,
            constants: constants
        }
        return service;

        function constants() {
            return {
                playbookTags: {
                    pBDesignerDescription: 'aibot-playbookPlanSuggestion',
                    pBDesignerSteps: 'aibot-playbookBlockSuggestion',
                    conversation: 'aibot-conversation'
                },
                configurationFailedMessage: "Unfortunately, there's not much that can be done at the moment. It seems that the OpenAI integration isn't properly configured or might be encountering issues. We recommend reaching out to your system administrator for assistance.",
                initialMessage: {
                    playbookSuggestionMessages: "Hi there! How can I help you today? I can help you generate playbook templates for your common use cases. For help on best practices on asking questions that give best results, use the 'help' icon above.",
                    conversationInDesigner: "Hi there! How can I help you today? Ask me questions like ‘how can I remove all HTML tags from a string’ OR ‘give some suggestions for responding to Ransomware alerts’ or ‘log samples for Fortigate firewall’.",
                    conversationThroughOutApplication: "Hi there! How can I help you today?"
                },
                connectorNotConfigured: 'Connector is not configured',
                connectorFetchApiFails: 'Error in fetching connector configuration',
                playbookFailed: "I'm sorry, it seems like there was an issue generating a response. Please try your request again, or refer to the advisor troubleshooting documentation for assistance.",
                playbookFailedToaster: 'Error in fetching playbook data',
                playbookSuggestionMessages: {
                    proceedingToGenerate: 'Proceeding to generate playbook template.',
                    generatedSuccessfully: 'Playbook template successfully generated! Adding the block now.',
                    buildingDescription: 'Sure, let me build and share a playbook outline to review.'
                }

            };
        }

        function connectorHealthCheck(connectorName) {
            return $resource('api/integration/connectors/?name=' + connectorName)
                .get()
                .$promise
                .then(function (connectorMetaDataForVersion) {
                    return connectorService.getConnector(connectorName, connectorMetaDataForVersion.data[0].version);
                })
                .then(function (connectorMetaData) {
                    if (connectorMetaData.configuration && connectorMetaData.configuration.length > 0) {
                        var promises = connectorMetaData.configuration.map(function (configs) {
                            return connectorService.getConnectorHealth(connectorMetaData, configs.config_id)
                                .then(function (configurationDetails) {
                                    return configurationDetails.status === 'Available';
                                });
                        });

                        // Use Promise.all to wait for all promises to resolve
                        return Promise.all(promises)
                            .then(function (results) {
                                // If any result is true, set connectorHealthCheckFlag to true
                                return results.some(Boolean);
                            });
                    } else {
                        return false;
                    }
                })
                .catch(function (error) {
                    console.error("Error in getConnector or getConnectorHealth:", error);
                    return false;
                });
        }


        function poll(_reqConfig, callback) {
            return $interval(function () {
                if (_reqConfig.hasValueReturned) { //check flag before start new call
                    callback(_reqConfig);
                }
                _reqConfig.thresholdValue = _reqConfig.thresholdValue - 1; //Decrease threshold value
                if (_reqConfig.thresholdValue === 0) {
                    stopPoll(_reqConfig); // Stop $interval if it reaches to threshold
                }
            }, _reqConfig.pollInterval);
        }
        function stopPoll(_reqConfig) {
            $interval.cancel(_reqConfig.pollPromise);
            _reqConfig.pollPromise = undefined;
            _reqConfig.thresholdValue = 0; //reset all flags.
            _reqConfig.hasValueReturned = true;
        }

        //If websocket is not active make api calls to check if the playbook execution is completed
        function checkPlaybookExecutionCompletionByPolling(params) {
            var _pollConfig = {
                hasValueReturned: true,
                thresholdValue: 150,
                pollInterval: params.pollInterval || 2000,
                pollPromise: undefined,
                defer: undefined
            };
            _pollConfig.defer = $q.defer();
            _pollConfig.pollPromise = poll(_pollConfig, function (callbackParam) {
                callbackParam.hasValueReturned = false;
                $resource(API.WORKFLOW + 'api/workflows/log_list/?format=json&parent__isnull=True&task_id=' + params.taskId).save({},
                    function (data) {
                        _pollConfig.hasValueReturned = true;
                        if (data['hydra:member'] && data['hydra:member'].length > 0 && (data['hydra:member'][0].status === 'finished' || data['hydra:member'][0].status === 'failed' || data['hydra:member'][0].status === 'terminated')) {
                            stopPoll(callbackParam);
                            _pollConfig.defer.resolve(data['hydra:member'][0]);
                        }
                    },
                    function (error) {
                        statusCodeService(error, true);
                        if (params.callback) {
                            params.callback(error);
                        }
                        stopPoll(callbackParam);
                        _pollConfig.defer.reject(error);
                    });
            });
            if (params.keepPollConfig) {
                playbookPollConfig.push(_pollConfig);
            }
            return _pollConfig.defer.promise;
        }

        //adds a listener to the triggered playbook
        function checkPlaybookExecutionCompletion(taskIds, callbackFunction, errorHandler, scope) {
            var defer = $q.defer();
            var processCompleted = 0;
            if ($rootScope.websocketConnected) {
                websocketService.subscribe('runningworkflow', function (result) {
                    if (scope.generatePlaybookFlag && (result.task_id && taskIds.includes(result.task_id) && result.status && (result.status === 'finished' || result.status === 'failed'))) {
                        processCompleted++;
                        updateProgressBar(scope, false, false, processCompleted);
                    }
                    if (result.task_id && taskIds.includes(result.task_id) && result.status && result.parent_wf === 'null' && (result.status === 'finished' || result.status === 'failed' || result.status === 'finished_with_error' || result.status === 'terminated')) {
                        taskIds.splice(taskIds.indexOf(result.task_id), 1);
                    }
                    if (taskIds.length === 0) {
                        callbackFunction(result);
                        websocketService.unsubscribe(scope.playbookStatusSubscription);
                        updateProgressBar(scope, false, true);
                        scope.playbookStatusSubscription = undefined;
                    }
                }).then(function (sub) {
                    playbookService.detachPaybookStatusWebsocket(scope.playbookStatusSubscription);
                    if (scope) {
                        scope.playbookStatusSubscription = sub;
                    }
                    defer.resolve(sub);
                }, function () {
                    defer.reject();
                    if (errorHandler) {
                        errorHandler();
                    }
                });
            }
            else {
                var taskIdsCopy = angular.copy(taskIds);
                angular.forEach(taskIdsCopy, function (taskId) {
                    if (scope.generatePlaybookFlag) {
                        updateProgressBar(scope, true, false);

                    }
                    checkPlaybookExecutionCompletionByPolling({ taskId: taskId }).then(function (response) {
                        if (response && (response.status === 'finished' || response.status === 'failed' || response.status === 'terminated')) {
                            taskIds.splice(taskIds.indexOf(taskId), 1);
                            response.instance_ids = parseInt(response['@id'].split('/wf/api/workflows/')[1]);
                        }
                        if (taskIds.length === 0) {
                            callbackFunction(response);
                            updateProgressBar(scope, false, true);
                        }
                    }, function () {
                        defer.reject();
                        if (errorHandler) {
                            errorHandler();
                        }
                    });
                });
            }
            return defer.promise;
        }

        //Update the loading percentage
        function updateProgressBar(scope, moveWithInterval, executionComplete, processCompleted) {
            if (executionComplete) {
                scope.processCompleted = 100;
                executionComplete = false;
            }
            else if (moveWithInterval) {
                var stepsExecuted = 1;
                var intervalId = setInterval(function () {
                    if (stepsExecuted < scope.numberOfDivisions) {
                        var chunk = Math.floor(((100 / scope.numberOfDivisions) * stepsExecuted));
                        if (chunk < 97) {
                            scope.processCompleted = chunk;
                        }
                        //If the number of steps in description and playbook block created are different
                        stepsExecuted++;
                    }
                    else {
                        clearInterval(intervalId);
                    }
                }, 30000);
            }
            else {
                var chunk = Math.floor(((100 / scope.numberOfDivisions) * processCompleted));
                //Check if the percentage is not greater than 100
                if (chunk < 97) {
                    scope.processCompleted = chunk;
                }
            }
        }

        //Copy all text to the local storage, without botgeneral
        function copyWithoutBotGeneral(originalData) {
            // Create a deep copy of the original data to avoid modifying the original object
            var copiedData = JSON.parse(JSON.stringify(originalData));

            // Filter out objects with type "botGeneral" from playbookSuggestionMessages except the first one
            copiedData.playbookSuggestionMessages = copiedData.playbookSuggestionMessages.filter(function (message, index) {
                return index === 0 || message.type !== "botGeneral";
            });

            // Filter out objects with type "botGeneral" from conversationInDesigner except the first one
            copiedData.conversationInDesigner = copiedData.conversationInDesigner.filter(function (message, index) {
                return index === 0 || message.type !== "botGeneral";
            });

            copiedData.conversationThroughOutApplication = copiedData.conversationThroughOutApplication.filter(function (message, index) {
                return index === 0 || message.type !== "botGeneral";
            });

            return copiedData;
        }

        function getAllPlaybooks(queryObject) {
            var defer = $q.defer();
            var url = 'api/query/workflows';
            $resource(url).save(queryObject, function (response) {
                if (response['hydra:member'] && (response['hydra:member'][0])) {
                    defer.resolve(response['hydra:member'][0]['uuid']);
                }
                else {
                    defer.reject("Playbook Not Found");
                }
            }, function (error) {
                defer.reject(error);
            });
            return defer.promise;
        }
    }
})();