/* 
Copyright start
Copyright (C) 2008 - 2023 Fortinet Inc.
All rights reserved.
FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
Copyright end 
*/
'use strict';
(function () {
  angular
    .module('cybersponse')
    .controller('aiAssistant100Ctrl', aiAssistant100Ctrl);

  aiAssistant100Ctrl.$inject = ['$scope', '$http', 'playbookService', 'toaster', '$rootScope', '$q', '$timeout', 'localStorageService', 'aiAssistantService', '$window', '$state', 'widgetBasePath'];

  function aiAssistant100Ctrl($scope, $http, playbookService, toaster, $rootScope, $q, $timeout, localStorageService, aiAssistantService, $window, $state, widgetBasePath) {

    var widgetBasePath = widgetBasePath;
    var constantMessages = aiAssistantService.constants();
    $scope.pageState = $state;
    $scope.updateRichTextHtmlValue = updateRichTextHtmlValue;
    $scope.clearConversation = clearConversation;
    $scope.generatePlaybook = generatePlaybook;
    $scope.generatePlaybookDescription = generatePlaybookDescription;
    $scope.onChangeMode = onChangeMode;
    $scope.openDocumentation = openDocumentation;
    $scope.playbookDescription = '';
    $scope.botType = 'Playbook Suggestion';
    $scope.showDescription = false;//To append review results button to the bot text, button only visible on playbook description
    $scope.generatePlaybookFlag = false;
    $scope.processingPlaybook = false;
    $scope.processingConversation = false;
    $scope.inPlaybookDesignerPage = false;
    $scope.disableTextArea = true;
    $scope.isConnectorConfigured = true;
    var stateChangeSuccess = '';
    $scope.isLightTheme = $rootScope.theme.id === 'light';//"widgets/installed/aiAssistant-1.0.0/"
    $scope.backgroundImageUrl = $scope.isLightTheme ? widgetBasePath + 'images/assistant-ui-white-background.svg' : widgetBasePath + 'images/assistant-ui-dark-background.svg';
    $scope.playbookTags = {
      pBDesignerDescription: [constantMessages.playbookTags.pBDesignerDescription],
      pBDesignerSteps: [constantMessages.playbookTags.pBDesignerSteps],
      conversation: [constantMessages.playbookTags.conversation]
    };
    $scope.options = {
      'name': 'Workflow Steps',
      mode: 'form',
      modes: ['code', 'form'],
      'enableTransform': true,
      'enableSort': false
    };
    $scope.configForMarkdown = {
      initialEditType: 'markdown',
      previewStyle: 'tab',
      height: '420px',
      usageStatistics: false,
      hideModeSwitch: true,
      toolbarItems: ['']
    };
    $scope.userInput = {
      textVal: ''
    };
    $scope.inputText = {
      inputText: '',
    };
    $scope.configurationFailedMessage = constantMessages.configurationFailedMessage;
    const staticMessage = {
      playbookSuggestionMessages: [{
        text: constantMessages.initialMessage.playbookSuggestionMessages,
        type: 'botGeneral'
      }],
      conversationInDesigner: [{
        text: constantMessages.initialMessage.conversationInDesigner,
        type: 'botGeneral'
      }],
      conversationThroughOutApplication: [{
        text: constantMessages.initialMessage.conversationThroughOutApplication,
        type: 'botGeneral'
      }]
    };
    $scope.messages = loadDataFromLocalStorage() || angular.copy(staticMessage);
    //variables for playbook percentage loader
    $scope.processCompleted = 6;


    init();
    function init() {
      $timeout(function () {
        scrollToBottom();
      })
      if ($scope.pageState.current.name === "main.playbookDetail") {
        $scope.inPlaybookDesignerPage = true;
      }
      stateChangeSuccessFunction();
      checkConnectorHealth();
    }

    function checkConnectorHealth() {
      if ($scope.disableTextArea) {
        aiAssistantService.connectorHealthCheck('openai').then(function (result) {
          if (result) {
            $scope.disableTextArea = false;
            $scope.isConnectorConfigured = true;
          }
          else {
            $scope.disableTextArea = true;
            $scope.isConnectorConfigured = false;
          }
        }, function (err) {
          $scope.disableTextArea = true;
          $scope.isConnectorConfigured = false;
        })
      }
      else {
        $scope.isConnectorConfigured = true;
        $scope.disableTextArea = false;
      }
    }

    function stateChangeSuccessFunction() {
      stateChangeSuccess = $scope.$on('$stateChangeSuccess', function (event, toState, toParams, from, fromParams) {
        // Condition for module listing page check
        if (toState.name.includes('main.playbookDetail')) {
          $scope.inPlaybookDesignerPage = true;
        }
        else {
          $scope.inPlaybookDesignerPage = false;
        }
      });
    }

    $scope.$on('popupOpened', function (event, data) {
      if (data === $scope.config.name + '_' + $scope.config.version) {
        $scope.isLightTheme = $rootScope.theme.id === 'light';
        $scope.backgroundImageUrl = $scope.isLightTheme ? widgetBasePath + 'images/assistant-ui-white-background.svg' : widgetBasePath + 'images/assistant-ui-dark-background.svg';
        checkConnectorHealth();
        if ($scope.pageState.current.name !== "main.playbookDetail") {
          $scope.inPlaybookDesignerPage = false;
        }
        else {
          $scope.inPlaybookDesignerPage = true;
        }
        stateChangeSuccessFunction();
      }
    })

    $scope.$on('popupClosed', function (event, data) {
      if (data === $scope.config.name + '_' + $scope.config.version) {
        //to destroy the listener for state change
        stateChangeSuccess();
      }
    })

    $scope.$on('$destroy', function () {
      stateChangeSuccess();
    });

    function loadDataFromLocalStorage() {
      return JSON.parse(localStorageService.get('aiAssistantBot.messages'));
    }

    function clearConversation() {
      if ($scope.inPlaybookDesignerPage) {
        if ($scope.botType === 'Conversation Playbook') {
          $scope.messages.conversationInDesigner = angular.copy(staticMessage.conversationInDesigner);
        }
        else {
          $scope.messages.playbookSuggestionMessages = angular.copy(staticMessage.playbookSuggestionMessages);
        }
      }
      else {
        $scope.messages.conversationThroughOutApplication = angular.copy(staticMessage.conversationThroughOutApplication);
      }
      updateLocalStorageData();
    }

    function updateLocalStorageData() {
      localStorageService.set('aiAssistantBot.messages', JSON.stringify(aiAssistantService.copyWithoutBotGeneral($scope.messages)));
    }

    //When radio button selection is changed
    function onChangeMode() {
      delete $scope.userInput.textVal;
      if ($scope.botType === 'Playbook Suggestion') {
        $scope.botType = 'Conversation Playbook';
      }
      else {
        $scope.botType = 'Playbook Suggestion';
      }
      $timeout(function () {
        scrollToBottom();
      })
    }

    //scroll to bottom when new conversation message is added
    function scrollToBottom() {
      var container = document.getElementById('bot-conversation');
      container.scrollTop = container.scrollHeight;
    }

    //generate response if enter is pressed in textbox
    $scope.sendMessage = function (event, flagEnterClick) {
      //if conversation option is selected
      if (((event.keyCode === 13 && !event.shiftKey) || flagEnterClick) && ($scope.userInput.textVal && $scope.userInput.textVal !== '')) {
        //Check if the bot is in designer page
        if ($scope.inPlaybookDesignerPage) {
          if ($scope.botType === 'Conversation Playbook') {
            // Check if Enter key is pressed
            $scope.messages.conversationInDesigner.push({ text: $scope.userInput.textVal, type: 'user' });
            updateLocalStorageData();
            $timeout(function () {
              scrollToBottom();
            })
            // Process user input and generate a bot response
            delete $scope.userInput.textVal;
            generatePBDesignerConversation($scope.messages.conversationInDesigner);
          }
          //when playbook suggestion is selected
          else {
            // Check if Enter key is pressed
            $scope.messages.playbookSuggestionMessages.push({ text: $scope.userInput.textVal, type: 'user' });
            updateLocalStorageData();
            var userInput = angular.copy($scope.userInput.textVal);
            $scope.userInput.textVal = ''; // Clear the input field
            // Process user input and generate a bot response
            generatePlaybookDescription(userInput);
          }
        }
        else {
          // Check if Enter key is pressed
          $scope.messages.conversationThroughOutApplication.push({ text: $scope.userInput.textVal, type: 'user' });
          updateLocalStorageData();
          $timeout(function () {
            scrollToBottom();
          })
          // Process user input and generate a bot response
          delete $scope.userInput.textVal;
          generateApplicationConversation($scope.messages.conversationThroughOutApplication);
        }
      }
    };

    // Function to generate a bot response 
    function generateApplicationConversation(userMessages) {
      $scope.processingConversation = true;
      var parametersForPlaybook = returnParameters();
      parametersForPlaybook.request.data['conversation'] = userMessages;
      parametersForPlaybook.request.data['botContext'] = $scope.pageState.current.name;

      //trigger playbook and get response when execution is completed
      getDataFromPlaybook($scope.playbookTags.conversation, parametersForPlaybook).then(function (data) {
        if (data.result && data.result.query_result && (data.status !== 'failed')) {
          $scope.messages.conversationThroughOutApplication.push({ text: data.result.query_result, type: 'bot' });
          updateLocalStorageData();
        }
        else {
          if ((data.status === 'failed')) {
            $scope.messages.conversationThroughOutApplication.push({ text: constantMessages.playbookFailed, type: 'bot' });
          }
        }
      },
        function (error) {
          $scope.messages.conversationThroughOutApplication.push({ text: constantMessages.playbookFailed, type: 'bot' });
        }
      ).finally(function () {
        $scope.processingConversation = false;
        $timeout(function () {
          scrollToBottom();
        })
      })
    }

    // Function to generate a bot response 
    function generatePBDesignerConversation(userMessages) {
      $scope.processingConversation = true;
      var parametersForPlaybook = returnParameters();
      parametersForPlaybook.request.data['conversation'] = userMessages;
      parametersForPlaybook.request.data['botContext'] = $scope.pageState.current.name;

      //trigger playbook and get response when execution is completed
      getDataFromPlaybook($scope.playbookTags.conversation, parametersForPlaybook).then(function (data) {
        if (data.result && data.result.query_result && (data.status !== 'failed')) {
          $scope.messages.conversationInDesigner.push({ text: data.result.query_result, type: 'bot' });
          updateLocalStorageData();
        }
        else {
          if ((data.status === 'failed')) {
            $scope.messages.conversationInDesigner.push({ text: constantMessages.playbookFailed, type: 'bot' });
          }
        }
      },
        function (error) {
          $scope.messages.conversationInDesigner.push({ text: constantMessages.playbookFailed, type: 'bot' });
        }
      ).finally(function () {
        $scope.processingConversation = false;
        $timeout(function () {
          scrollToBottom();
        })
      })
    }

    //when review button is clicked paste the playbook block on pb-designer
    function generatePlaybook(descriptionMessage) {
      var playbookDescription = (typeof descriptionMessage === 'object' && descriptionMessage !== null) ? descriptionMessage : JSON.parse(descriptionMessage);
      $scope.messages.playbookSuggestionMessages.push({ text: constantMessages.playbookSuggestionMessages.proceedingToGenerate, type: 'botGeneral' });
      $timeout(function () {
        scrollToBottom();
      })

      var connectorsNotInstalled = [];
      var parametersForPlaybook = returnParameters();
      parametersForPlaybook.request.data['task_to_automate'] = playbookDescription;
      parametersForPlaybook.request.data['botContext'] = $scope.pageState.current.name;
      $scope.numberOfDivisions = countStepKeys(playbookDescription);
      $scope.processingPlaybook = true;
      $scope.generatePlaybookFlag = true;
      scrollToBottom();
      //trigger playbook and get response when execution is completed
      getDataFromPlaybook($scope.playbookTags.pBDesignerSteps, parametersForPlaybook).then(function (data) {
        $scope.executionCompleted = true;
        $scope.generatePlaybookFlag = false;
        $scope.processCompleted = 6;
        if (data.result && data.result.data && (data.status !== 'failed')) {
          $rootScope.$broadcast('designer:addPlaybookElements', data.result.data);
          const customModal = document.getElementById('custom-modal');
          $timeout(function () {
            customModal.setAttribute('style', 'display:none;');
          }, 6000);
          toaster.info('Copied all selected steps');
          if ($scope.botType === 'Playbook Suggestion' && data.status != 'failed') {
            $scope.messages.playbookSuggestionMessages.push({
              text:
                constantMessages.playbookSuggestionMessages.generatedSuccessfully, type: 'botGeneral'
            });
            if (data.result.data.connectors_not_installed && data.result.data.connectors_not_installed.length > 0) {
              connectorsNotInstalled = data.result.data.connectors_not_installed.map(item => item.label);
              $scope.messages.playbookSuggestionMessages.push({ connectorsNotInstalled: connectorsNotInstalled, type: 'botGeneral' });
            }
            $timeout(function () {
              scrollToBottom();
            });
          }
        }
        else {
          $scope.messages.playbookSuggestionMessages.push({ text: constantMessages.playbookFailed, type: 'bot' });
        }
      },
        function (error) {
          $scope.messages.playbookSuggestionMessages.push({ text: constantMessages.playbookFailed, type: 'bot' });
        }
      ).finally(function () {
        $scope.processingPlaybook = false;
        $timeout(function () {
          scrollToBottom();
        });
        setTimeout(function () {
          $rootScope.isBotWidgetOpen = false;
          $scope.close();
        }, connectorsNotInstalled.length < 1 ? 3000 : 5000);
      });
    }

    //get JSON playbook description 
    function generatePlaybookDescription(inputText) {
      $scope.processingPlaybook = true;
      var generateButton = document.getElementById('generate-result-button');
      if (generateButton) {
        generateButton.style.display = 'none';
      }
      var parametersForPlaybook = returnParameters();
      parametersForPlaybook.request.data['botContext'] = $scope.pageState.current.name;
      parametersForPlaybook.request.data['task_to_automate'] = inputText;
      if ($scope.botType === 'Playbook Suggestion') {
        $scope.messages.playbookSuggestionMessages.push({ text: constantMessages.playbookSuggestionMessages.buildingDescription, type: 'botGeneral' });
        $timeout(function () {
          scrollToBottom();
        })
      }
      getDataFromPlaybook($scope.playbookTags.pBDesignerDescription, parametersForPlaybook).then(function (data) {
        if (data.result && data.result.query_result && data.result.query_result.result && data.status !== 'failed') {
          $scope.processingPlaybook = false;
          $scope.messages.playbookSuggestionMessages.push({ text: data.result.query_result.result, type: 'bot', generatePlaybook: true });
          updateLocalStorageData();
          $scope.playbookDescription = data.result.query_result.result;
          $timeout(function () {
            scrollToBottom();
          })
        }
        else {
          $scope.processingPlaybook = false;
          $scope.playbookFailed = true;
          toaster.error({ body: constantMessages.playbookFailedToaster });
        }
      }
      ).catch(
        function (error) {
          $scope.processingPlaybook = false;
          $scope.playbookFailed = true;
          $scope.messages.playbookSuggestionMessages.push({ text: constantMessages.playbookFailed, type: 'botGeneral' });
        })
    }

    //trigger respective playbook to either get playbook of conversation text
    function getDataFromPlaybook(playbookTags, parametersForPlaybook) {
      var defer = $q.defer();
      var queryObjectPlaybook = {
        'sort': [
          {
            'field': 'createDate',
            'direction': 'DESC',
            '_fieldName': 'createDate'
          }
        ],
        'limit': 1,
        'logic': 'AND',
        'filters': [
          {
            'field': 'recordTags',
            'value': playbookTags,
            'display': '',
            'operator': 'in',
            'type': 'array'
          },
          {
            'field': 'isActive',
            'value': true,
            'operator': 'eq'
          }
        ],
        '__selectFields': [
          'id',
          'name'
        ]
      };
      $scope.processCompleted = 6;
      // add a tag to the playbook, get the playbook instead of hardcoding the playbook iri
      aiAssistantService.getAllPlaybooks(queryObjectPlaybook).then(function (playbookUUID) {
        var queryUrl = '/api/triggers/1/notrigger/' + playbookUUID + '?force_debug=true';
        //Parametersforplaybook -> 'Edit Parameters' in playbook
        $http.post(queryUrl, parametersForPlaybook).then(function (result) {
          if (result && result.data && result.data.task_id) {
            aiAssistantService.checkPlaybookExecutionCompletion([result.data.task_id], function (response) {
              if (response && (response.status === 'finished' || response.status === 'failed' || response.status === 'terminated')) {
                playbookService.getExecutedPlaybookLogData(response.instance_ids).then(function (res) {
                  defer.resolve(res);
                }, function (err) {
                  defer.reject(err);
                });
              }
            }, function (error) {
              defer.reject(error);
            }, $scope);
          }
        }, function (err) {
          defer.reject(err);
        });
      }, function (err) {
        defer.reject(err);
      });
      return defer.promise;
    }

    function returnParameters() {
      return {
        'input': {},
        'request': {
          'data': {
            'records': [],
          }
        },
        'useMockOutput': false,
        'globalMock': false,
        'force_debug': true
      };
    }

    function countStepKeys(data) {
      let count = 0;

      function countKeysRecursively(subObj) {
        for (const key in subObj) {
          if (key.includes('Step')) {
            count++;
          }
          if (typeof subObj[key] === 'object') {
            countKeysRecursively(subObj[key]);
          }
        }
      }
      countKeysRecursively(data);
      return count;
    }

    function openDocumentation() {
      $window.open('https://github.com/fortinet-fortisoar/solution-pack-fortinet-advisor/blob/release/1.0.0/docs/usage.md#prompting-tips', '_blank');
    }

    function updateRichTextHtmlValue(value) {
      //Placeholder
      updateLocalStorageData();
      return;
    }
  }
})();
